import { co2 } from "https://cdn.skypack.dev/@tgwf/co2";

(function ($, Drupal, drupalSettings) {
    'use strict';
    Drupal.behaviors.carbonevaluatoronebytemodel = {
        attach: function (context) {
            /**
             * When the body has finished loading, it checks that 
             * the Per Byte model is active and calculates the CO2 
             * footprint per Byte of the current page.
             * 
             * This footprint is expressed in gCO2/Per Byte.
             */
            $(once('bodyloadperbyte', 'body')).each(function (e) {
                if (drupalSettings.per_byte == 1) {
                    const oneByte = new co2({
                        model: "1byte",
                        results: "segment",
                        version: 4
                    });
                    let pageLoadSize = getPageSize();
                    const greeHost = true;
                    var perByte = oneByte.perByte(pageLoadSize, greeHost);
                    $('body').find('.co2-per-byte i').html(perByte + ' gCO2/Per byte');
                    sendPervisitData(perByte, 'per_byte');
                }
            });
        }
    }
    Drupal.behaviors.carbonevaluatorwebdesingmodel = {
        attach: async function (context) {
            /**
             * When the body has finished loading, it checks that the Per Visit 
             * model is active and calculates the CO2 footprint per Visit 
             * of the current page. 
             * 
             * This footprint is expressed in gCO2/Per Visit.
             */
            $(once('bodyloadpervisit', 'body')).each(function (e) {
                if (drupalSettings.per_visit == 1) {
                    let nid = drupalSettings.nid;
                    var visitStorage = localStorage.getItem(nid);
                    var visitStatus = "";
                    var reloadSize = "";
                    if (visitStorage == null) {
                        visitStatus = 'first_visit';
                        localStorage.setItem(nid, visitStatus);
                        let pageSize = getPageSize();
                        localStorage.setItem(nid + '_size', pageSize);
                        reloadSize = 0;
                    } else {
                        visitStatus = 'return_visit';
                        localStorage.setItem(nid, visitStatus);
                        reloadSize = getPageSize();
                    }
                    sendRequest(visitStatus, reloadSize, drupalSettings);
                }
            });
        }
    }

    /**
     * It creates an Ajax request of the POST type which is then received in the module's backoffice.
     * This request sends the page ID and the status of the visit.
     * 
     * @param {visitStatus} visitStatus - It checks whether it's the first time the user has visited that page or not.
     * @param {reloadSize} reloadSize - India the number of bytes when the page is reloaded.
     * @param {drupalSettings} drupalSettings - It mentions all the values stored in the Drupal settings.
     */
    function sendRequest(visitStatus, reloadSize, drupalSettings) {
        let nid = drupalSettings.nid;
        const hostname = location.hostname;
        $.ajax({
            url: 'https://' + hostname + '/carbon-impact-evaluator/visits',
            method: 'POST',
            data: {
                valor: JSON.stringify({
                    'nid': nid,
                    'visit_status': visitStatus
                })
            },
            success: function (response) {
                getCO2(response, reloadSize);
            },
            error: function (error) {
                console.error(Drupal.t('Erro ao enviar o valor:'), error);
            }
        });
    }

    /**
     * 
     * @returns The page size.
     */
    function getPageSize() {
        const resources = performance.getEntriesByType('resource');
        var resourcesSize = 0;
        resources.forEach(element => {
            resourcesSize += element.transferSize;
        });
        const navigation = performance.getEntriesByType('navigation')[0];
        let navigationSize = navigation.transferSize;
        let totalSize = resourcesSize + navigationSize;
        return totalSize;
    }

    /**
     * It works on the response to the Ajax POST request created in the 
     * function and calculates the carbon footprint.
     * 
     * @param {response} response - Ajax POST request response.
     * @param {reloadSize} reloadSize - Number of bytes of the page reload.
     */
    function getCO2(response, reloadSize) {
        let nid = drupalSettings.nid;
        let first = response.valor.first_visit;
        let total = response.valor.total_number_visits;
        let returnV = response.valor.return_visit;
        // 2) Vamos fazer os cálculos
        let firstPercentage = parseFloat(((first * 100) / total) / 100);
        let returnPercentage = parseFloat((returnV * 100) / total / 100);
        // Uses the most recent version for this model (v0.16.1). The default is version 3 (v0.11.0).
        const carbonEmissions = new co2({
            model: "swd",
            version: 4
        });
        let pageSize = localStorage.getItem(nid + '_size');
        var reloadRatio = (((reloadSize * 100) / pageSize) / 100);
        const greeHost = drupalSettings.Greenhost;
        let datacenterCountry = drupalSettings.Datacenter_country;
        const options = {
            dataReloadRatio: parseFloat(reloadRatio),
            firstVisitPercentage: parseFloat(firstPercentage),
            returnVisitPercentage: parseFloat(returnPercentage),
            gridIntensity: {
                dataCenter: { country: datacenterCountry },
            },
        };
        var calculatedCarbonEmissions = carbonEmissions.perVisitTrace(pageSize, greeHost, options).co2;
        $('body').find('.co2-per-visit i').html(calculatedCarbonEmissions + ' gCO2/Per visit');
        sendPervisitData(calculatedCarbonEmissions, 'per_visit');
    }

    /**
     * Sends the calculated Per Visit values to the module's back office so that they can be stored in the table.
     * 
     * @param {co2Calculations} co2Calculations - The result of the Per Visit/Per Byte calculations.
     * @param {identifier} identifier - Model identifier.
     */
    function sendPervisitData(co2Calculations, identifier) {
        let nid = drupalSettings.nid;
        const hostname = location.hostname;
        $.ajax({
            url: 'https://' + hostname + '/carbon-impact-evaluator/pervisit',
            method: 'POST',
            data: {
                valor: JSON.stringify({
                    'nid': nid,
                    'identifier': identifier,
                    'calculations': co2Calculations
                })
            },
            success: function (response) {
                console.log(Drupal.t('Sucesso ao enviar o pedido:'), response);
            },
            error: function (error) {
                console.error(Drupal.t('Erro ao enviar o valor:'), error);
            }
        });
    }

})(jQuery, Drupal, drupalSettings);